# from unittest import TestCase
#
# from aiuna.step.dataset import dataset
# from tatu import Tatu
# from flask import Flask
#
# from flask_sqlalchemy import SQLAlchemy
#
# class TestSQLA(TestCase):
#     def insert(self):
#         app = Flask(__name__)
#         db = SQLAlchemy(app)
#         db =??
#         tatu = Tatu(db)
#
#         d = dataset.data
